import argparse
from looker_deployer.commands import deploy_boards, deploy_code, deploy_connections, deploy_content
from looker_deployer import __version__ as pkg


loc = "looker.ini"


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--version", action="store_true", help="print version and exit"),
    subparsers = parser.add_subparsers()
    setup_board_subparser(subparsers)
    setup_code_subparser(subparsers)
    setup_connections_subparser(subparsers)
    setup_content_subparser(subparsers)
    args = parser.parse_args()
    if args.version:
        print(pkg.__version__)
        parser.exit(0)
    else:
        try:
            args.func(args)
        except AttributeError:
            parser.print_help()
            parser.exit(1)


def setup_board_subparser(subparsers):
    boards_subparser = subparsers.add_parser("boards")
    boards_subparser.add_argument("--source", required=True, help="which environment to source the board from")
    boards_subparser.add_argument("--target", required=True, nargs="+", help="which target environment(s) to deploy to")
    boards_subparser.add_argument("--board", required=True, help="which board to deploy")
    boards_subparser.add_argument("--ini", default=loc, help="ini file to parse for credentials")

    boards_subparser.add_argument(
        "--title-change",
        help="if updating title, the old title to replace in target environments"
    )

    boards_subparser.add_argument("--debug", action="store_true", help="set logger to debug for more verbosity")
    boards_subparser.set_defaults(func=deploy_boards.main)


def setup_code_subparser(subparsers):
    code_subparser = subparsers.add_parser("code")
    code_subparser.add_argument("--hub", action="store_true", help="flag to deploy hub project")
    code_subparser.add_argument("--spoke", nargs='+', help="which spoke(s) to deploy")
    code_subparser.add_argument("--hub-exclude", nargs="+", help="which projects should be ignored from hub deployment")
    code_subparser.add_argument("--debug", action="store_true", help="set logger to debug for more verbosity")
    code_subparser.set_defaults(func=deploy_code.main)


def setup_connections_subparser(subparsers):
    connections_subparser = subparsers.add_parser("connections")
    connections_subparser.add_argument("--source", required=True, help="which environment to source the board from")
    connections_subparser.add_argument("--ini", default=loc, help="ini file to parse for credentials")
    connections_subparser.add_argument(
        "--target",
        required=True,
        nargs="+",
        help="which target environment(s) to deploy to"
    )
    connections_subparser.add_argument("--pattern", help="regex pattern to filter which connections are deployed")
    connections_subparser.add_argument(
        "--include-password",
        action="store_true",
        help="should passwords be set from the ini file?"
    )
    connections_subparser.add_argument("--debug", action="store_true", help="set logger to debug for more verbosity")
    connections_subparser.set_defaults(func=deploy_connections.main)


def setup_content_subparser(subparsers):
    content_subparser = subparsers.add_parser("content")
    content_subparser.add_argument("--env", required=True, help="What environment to deploy to")
    content_subparser.add_argument("--ini", default=loc, help="ini file to parse for credentials")
    content_subparser.add_argument("--debug", action="store_true", help="set logger to debug for more verbosity")
    content_subparser.add_argument("--recursive", action="store_true", help="Should folders deploy recursively")
    content_subparser.add_argument("--target-folder", help="override the default target folder with a custom path")
    content_group = content_subparser.add_mutually_exclusive_group(required=True)
    content_group.add_argument("--folders", nargs="+", help="Folders to fully deploy")
    content_group.add_argument("--dashboards", nargs="+", help="Dashboards to deploy")
    content_group.add_argument("--looks", nargs="+", help="Looks to deploy")
    content_group.add_argument("--export", help="pull content from dev")
    content_subparser.set_defaults(func=deploy_content.main)



import logging
from looker_sdk import models
from looker_deployer.utils import deploy_logging
from looker_deployer.utils.get_client import get_client

logger = deploy_logging.get_logger(__name__)


def match_dashboard_id(source_dashboard_id, source_sdk, target_sdk):
    source = source_sdk.dashboard(str(source_dashboard_id))
    logger.debug("Attempting dashboard match", extra={"title": source.title, "slug": source.slug, "id": source.id})
    target_dash = target_sdk.search_dashboards(slug=source.slug)

    assert len(target_dash) < 2, "Found multiple slugs. Something is wrong and it's probably time to contact Looker"
    assert len(target_dash) == 1, "Could not find dashboard {source.title} in target environment. Has it been deployed?"

    target_id = target_dash[0].id
    logger.debug("Found dashboard", extra={"id": target_id})

    return target_id


def match_look_id(source_look_id, source_sdk, target_sdk):
    source = source_sdk.look(source_look_id)
    logger.debug("Attempting look match", extra={"title": source.title, "id": source.id})
    target_look = target_sdk.search_looks(title=source.title)

    assert len(target_look) < 2, "Found multiple titles. Please rename a look or remove duplicates"
    assert len(target_look) == 1, "Could not find look {source.title} in target environment. Has it been deployed?"

    target_id = target_look[0].id
    logger.debug("Found look", extra={"id": target_id})

    return target_id


def return_board(board_name, source_sdk):
    logger.debug("Searching boards", extra={"title": board_name})
    board_list = source_sdk.search_homepages(title=board_name)
    assert len(board_list) < 2, "More than one board found! Refine your search or remove duplicate names."
    assert len(board_list) == 1, "Could not find board! Double check available titles and try again."

    logger.debug("Found board", extra={"board": board_list})
    return board_list[0]


def create_or_update_board(source_board_object, target_sdk, title_override=None):

    # Determine if board already exists in target environment
    search_title = title_override or source_board_object.title
    search_res = target_sdk.search_homepages(title=search_title)
    assert len(search_res) < 2, "More than one board found! Refine your search or remove duplicate names."

    try:
        assert len(search_res) == 1

    # If board does not exist then create
    except AssertionError:
        logger.info(
            "No pre-existing board found. Creating new board in target environment",
            extra={"title": search_title}
        )

        new_board = models.WriteHomepage(
            title=source_board_object.title,
            description=source_board_object.description
        )

        resp = target_sdk.create_homepage(new_board)
        logger.info("Board created", extra={"id": resp.id})
        return resp.id

    # If board already exists, clear out sections and update
    logger.info(
        "Found board in target instance. Updating and rebuilding content",
        extra={"title": search_title}
    )

    target_board = search_res[0]

    # Clear out existing sections
    section_list = [i.id for i in target_board.homepage_sections]
    logger.debug("Found sections to clear", extra={"section_list": section_list})

    for section in section_list:
        logger.debug("Clearing section for refresh", extra={"section_id": section})
        target_sdk.delete_homepage_section(section)

    # Update
    update_board = models.WriteHomepage(
        title=source_board_object.title,
        description=source_board_object.description
    )

    resp = target_sdk.update_homepage(target_board.id, update_board)
    logger.info("Board updated", extra={"id": resp.id})
    return resp.id


def create_board_section(source_board_section_object, target_board_id, target_sdk):
    new_board_section = models.WriteHomepageSection(
        title=source_board_section_object.title,
        description=source_board_section_object.description,
        homepage_id=target_board_id
    )

    logger.info("Creating Section", extra={"board_id": target_board_id, "section_title": new_board_section.title})
    resp = target_sdk.create_homepage_section(new_board_section)
    logger.info("Section created", extra={"section_id": resp.id})
    return resp.id


def create_board_item(source_board_item_object, target_board_section_id, source_sdk, target_sdk):

    dashboard_id = None
    look_id = None

    if source_board_item_object.dashboard_id:
        dashboard_id = match_dashboard_id(source_board_item_object.dashboard_id, source_sdk, target_sdk)
    if source_board_item_object.look_id:
        look_id = match_look_id(source_board_item_object.look_id, source_sdk, target_sdk)

    new_board_item = models.WriteHomepageItem()
    new_board_item.__dict__.update(source_board_item_object.__dict__)
    new_board_item.dashboard_id = dashboard_id
    new_board_item.look_id = look_id
    new_board_item.homepage_section_id = target_board_section_id

    logger.info(
        "Creating item",
        extra={
            "section_id": new_board_item.homepage_section_id,
            "dashboard_id": new_board_item.dashboard_id,
            "look_id": new_board_item.look_id,
            "url": new_board_item.url
        }
    )
    resp = target_sdk.create_homepage_item(new_board_item)
    logger.info("Item created", extra={"id": resp.id})

    return resp


def audit_board_content(board_object, source_sdk, target_sdk):
    dash_list = []
    look_list = []

    for i in board_object.homepage_sections:
        for j in i.homepage_items:
            if j.dashboard_id:
                dash_list.append(j.dashboard_id)
            if j.look_id:
                look_list.append(j.look_id)

    for dash in dash_list:
        match_dashboard_id(dash, source_sdk, target_sdk)
    for look in look_list:
        match_look_id(look, source_sdk, target_sdk)


def send_boards(board_name, source_sdk, target_sdk, title_override=None):
    source_board = return_board(board_name, source_sdk)
    audit_board_content(source_board, source_sdk, target_sdk)
    target_board_id = create_or_update_board(source_board, target_sdk, title_override)

    for section in source_board.homepage_sections:
        target_section_id = create_board_section(section, target_board_id, target_sdk)

        for item in section.homepage_items:
            create_board_item(item, target_section_id, source_sdk, target_sdk)


def main(args):

    if args.debug:
        logger.setLevel(logging.DEBUG)

    source_sdk = get_client(args.ini, args.source)

    for t in args.target:
        target_sdk = get_client(args.ini, t)

        send_boards(args.board, source_sdk, target_sdk, args.title_change)



import requests
import oyaml as yaml
import logging
from requests import ConnectionError
from looker_deployer.utils import deploy_logging
from looker_deployer.utils import parse_ini

logger = deploy_logging.get_logger(__name__)


def get_secret(target):
    webhooks = parse_ini.read_ini("./looker.ini")["Webhooks"]
    logger.info("Fetching secret", extra={"target": target})
    secret = webhooks[f"looker_{target.lower()}_deploy_secret"]
    return {"X-Looker-Deploy-Secret": secret}


def parse_hub_endpoints(config):
    instances = config["instances"]
    if config.get("hub_deploy_exclude"):
        logger.info("Detected exclude list", extra={"excluded": config.get("hub_deploy_exclude")})
        excludes = config.get("hub_deploy_exclude")
        endpoints = [i["endpoint"] for i in instances if i["name"] not in excludes]
    else:
        endpoints = [i["endpoint"] for i in instances]

    logger.info("Parsed endpoints", extra={"endpoints": endpoints})
    return endpoints


def parse_spoke_config(spoke_name, config):
    spoke_config = [i for i in config["instances"] if i["name"] == spoke_name][0]
    logger.info("Parsed spoke config", extra={"config": spoke_config})

    return spoke_config


def parse_hub_excludes(config, arg=None):
    if arg and type(config.get("hub_deploy_exclude")) == list:
        config["hub_deploy_exclude"] += arg
    elif arg:
        config["hub_deploy_exclude"] = arg


def deploy_code(project, endpoint, header):
    deploy_url = "/".join(
        [
            endpoint,
            "webhooks",
            "projects",
            project,
            "deploy"
        ]
    )

    logger.info("Deploying", extra={"project": project, "deploy_url": deploy_url})

    try:
        r = requests.get(deploy_url, headers=header)
        assert r.status_code == 200, "Bad response code"
    except ConnectionError as e:
        logger.error("URL Not Found - check %s's config endpoint for typos?", project)
        raise e
    except AssertionError as e:
        if r.status_code == 500:
            logger.error("Bad authorization attempt - check environment variable settings for webhook deploy secret")
            raise e
        else:
            logger.error("Unknown Error: %s", str(r.json()))
            raise e
    results = r.json()["operations"][0]["results"][0]
    logger.info("Deployment complete. Status: %s", results)
    return r.json()


def main(args):

    if args.debug:
        logger.setLevel(logging.DEBUG)

    with open("./code_config.yaml") as f:
        config = yaml.safe_load(f)

    if args.hub:
        project = config["hub_project"]
        target = "hub"

        parse_hub_excludes(config, args.hub_exclude)

        endpoints = parse_hub_endpoints(config)
        auth_header = get_secret(target)

        for endpoint in endpoints:
            logger.info("Deploying hub to %s", endpoint)
            deploy_code(project, endpoint, auth_header)

    if args.spoke:

        for i in args.spoke:
            try:
                spoke_config = parse_spoke_config(i, config)
            except IndexError:
                logger.error("Invalid name %s. Skipping...", i)
                continue

            project = spoke_config["spoke_project"]
            target = spoke_config["name"]
            endpoint = spoke_config["endpoint"]
            auth_header = get_secret(target)

            deploy_code(project, endpoint, auth_header)



from setuptools import setup
from looker_deployer import __version__ as pkg

NAME = "looker_deployer"
VERSION = pkg.__version__
REQUIRES = ["looker-sdk==0.1.3b14", "oyaml", "python-json-logger"]

setup(
    author="Colin Pistell",
    author_email="colin.pistell@looker.com",
    description="A Looker Deployment Tool",
    install_requires=REQUIRES,
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords=["Looker Deployer"],
    license="MIT",
    name=NAME,
    packages=["looker_deployer", "looker_deployer/commands", "looker_deployer/utils"],
    entry_points={"console_scripts": ["ldeploy=looker_deployer.cli:main"]},
    python_requires=">=3.6.0, <3.9",
    version=VERSION
)
